@@ -4,7 +4,7 @@
 import net_genconfig
 
 import argparse
-from copy import deepcopy
+from copy import copy, deepcopy
 import datetime
 import jinja2
 import os
@@ -452,7 +452,7 @@ for filter_name, filter_func in (
 
 
 
-def profiles_merge(source_description, device):
+def profiles_merge(source_description, device, included_profiles=set()):
     """This function recursively merges profiles into the supplied
     device (or device profile).
     
@@ -479,6 +479,11 @@ def profiles_merge(source_description, device):
     device -- a device or device profile definition dictionary that is
     used as the definition to merge into, as well as read the
     "profiles" and "profiles-exclude" keys from (if present).
+
+    included_profiles=set() -- the set of profiles which have been
+    included so far - this is used to identify recursive profile usage
+    and abort with a helpful error (rather than Python crashing with a
+    recursion error).
     """
 
     # this function makes use of global variables defined outside it so
@@ -531,9 +536,38 @@ def profiles_merge(source_description, device):
 
                   file=sys.stderr)
 
+
+        # check if we've included this profile already, at some point higher 
+        # in the hierarchy - if we have, abort with an error
+
+        if profile in included_profiles:
+            print("error: %s recursive inclusion of device profile: %s"
+                      % (source_description, profile),
+
+                  file=sys.stderr)
+
+            exit(1)
+
+
+        # copy the included profiles set and add this profile to it, before
+        # doing the recursive profile merge
+        #
+        # doing this here will allow the same profile to be included twice, in
+        # different branches of the hierarchy, which we want to allow, as they
+        # will be in a different place in the order and may have different
+        # exclusion definitions
+        #
+        # this will also allow the same profile to be included multiple times
+        # by the same device or device profile, which doesn't make sense but
+        # is not an error, so we can let it go
+
+        included_profiles_this = copy(included_profiles)
+        included_profiles_this.add(profile)
+
         sub_profile_merged = profiles_merge(
-                                    "device profile: %s" % profile,
-                                    inventory["device-profiles"][profile])
+                                 "device profile: %s" % profile,
+                                 inventory["device-profiles"][profile],
+                                 included_profiles_this)
 
 
         # merge a deepcopy() of this profile into the composite profile
